/*
 * This file is part of Loctrace.
 * 
 * Copyright (c) 2007 Thomas King <king@informatik.uni-mannheim.de>,
 * University of Mannheim, Germany
 * 
 * All rights reserved.
 *
 * Loclib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Loclib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Loclib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package org.pi4.loctrace;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Point2D;
import java.io.File;
import java.io.IOException;
import java.util.List;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.pi4.loctrace.util.PositionFileParser;

/**
 * Configuration panel for the position.
 * 
 * @author Andreas
 */
@SuppressWarnings("serial")
public class PositionPanel extends JPanel {
	
	private JSpinner positionX, positionY, positionStep;
	private JTextField angle, positionFile;
	private JButton positionFileButton, previousPositionButton, nextPositionButton;
	protected JPanel self = this;
	
	// model
	private List<Point2D> positions;
	private int positionIndex;
	
	public PositionPanel() {
		this.setBorder(BorderFactory.createTitledBorder("Position"));
		GridBagLayout positionLayout = new GridBagLayout();
		this.setLayout(positionLayout);
		
		final int h = 24;
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.gridx = 0;
		constraints.gridy = 0;
		constraints.weightx = 0.0;
		constraints.weighty = 1.0;
		constraints.insets.set(8, 8, 0, 8);
		constraints.anchor = GridBagConstraints.WEST;
		JLabel label = new JLabel("X:");
		this.add(label, constraints);
		constraints.gridx++;
		//constraints.weightx = 1.0;
		constraints.insets.set(5, 0, 0, 8);
		positionX = new JSpinner(new SpinnerNumberModel(1.0, 0.0, Double.POSITIVE_INFINITY, 0.5));
		positionX.setPreferredSize(new Dimension(60, h));
		this.add(positionX, constraints);
		constraints.gridy++;
		constraints.gridx = 0;
		//constraints.weightx = 0.0;
		constraints.insets.set(8, 8, 0, 8);
		label = new JLabel("Y:");
		this.add(label, constraints);
		constraints.gridx++;
		//constraints.weightx = 1.0;
		constraints.insets.set(5, 0, 0, 8);
		positionY = new JSpinner(new SpinnerNumberModel(1.0, 0.0, Double.POSITIVE_INFINITY, 0.5));
		positionY.setPreferredSize(positionX.getPreferredSize());
		this.add(positionY, constraints);
		constraints.gridx++;
		constraints.gridy--;
		constraints.gridheight = 2;
		constraints.insets.set(8, 8, 0, 8);
		label = new JLabel("Step size: ");
		this.add(label, constraints);
		constraints.gridx++;
		constraints.insets.set(5, 0, 0, 8);
		positionStep = new JSpinner(new SpinnerNumberModel(1.0, 0.0, Double.POSITIVE_INFINITY, 0.1));
		positionStep.setPreferredSize(new Dimension(60, h));
		positionStep.addChangeListener(new PositionStepController());
		this.add(positionStep, constraints);
		
		constraints.gridy += 2;
		constraints.gridx = 0;
		constraints.gridheight = 1;
		//constraints.weightx = 0.0;
		constraints.insets.set(8, 8, 0, 8);
		label = new JLabel("Angle: ");
		this.add(label, constraints);
		constraints.gridx++;
		//constraints.weightx = 1.0;
		//constraints.fill = GridBagConstraints.HORIZONTAL;
		constraints.insets.set(5, 0, 0, 8);
		angle = new JTextField(5);
		//angle.setSize(50, angle.getHeight());
		angle.setEditable(false);
		this.add(angle, constraints);
		
		constraints.gridy++;
		constraints.gridx = 0;
		constraints.insets.set(8, 8, 0, 8);
		label = new JLabel("File:");
		this.add(label, constraints);
		constraints.gridx++;
		constraints.gridwidth = 3;
		constraints.insets.set(5, 0, 0, 8);
		positionFile = new JTextField(20);
		positionFile.setEditable(false);
		this.add(positionFile, constraints);
		constraints.gridwidth = 1;
		
		constraints.gridx += 3;
		positionFileButton = new JButton("Choose...");
		positionFileButton.addActionListener(new PositionFileButtonController());
		this.add(positionFileButton, constraints);
		
		constraints.gridx++;
		constraints.weightx = 1.0;
		constraints.insets.set(5, 0, 0, 0);
		JPanel panel = new JPanel();
		panel.setLayout(new FlowLayout(FlowLayout.LEADING, 5, 0));
		previousPositionButton = new JButton("< Previous");
		previousPositionButton.addActionListener(new PositionButtonController());
		panel.add(previousPositionButton);
		
		constraints.gridx++;
		constraints.weightx = 1.0;
		nextPositionButton = new JButton("Next >");
		nextPositionButton.addActionListener(new PositionButtonController());
		panel.add(nextPositionButton);
		this.add(panel, constraints);
	}
	
	public void setEnabled(boolean enabled) {
		super.setEnabled(enabled);
		positionX.setEnabled(enabled);
		positionY.setEnabled(enabled);
		positionStep.setEnabled(enabled);
		angle.setEnabled(enabled);
		positionFile.setEnabled(enabled);
		positionFileButton.setEnabled(enabled);
		if (enabled) {
			updatePositionButtons();
		} else {
			nextPositionButton.setEnabled(false);
			previousPositionButton.setEnabled(false);
		}
	}
	
	
	/**
	 * Controller for position step size.
	 */
	private class PositionStepController implements ChangeListener {

		public void stateChanged(ChangeEvent e) {
			Double stepSize = (Double)positionStep.getValue();
			SpinnerNumberModel model;
			model = (SpinnerNumberModel) positionX.getModel();
			model.setStepSize(stepSize);
			model = (SpinnerNumberModel) positionY.getModel();
			model.setStepSize(stepSize);
		}
		
	}
	
	/**
	 * Controller for position file button.
	 */
	private class PositionFileButtonController implements ActionListener {
		
		public void actionPerformed(ActionEvent e) {
			JFileChooser fileChooser = new JFileChooser();
			if (fileChooser.showOpenDialog(self) == JFileChooser.APPROVE_OPTION) {
				try {
					setPositionFile(fileChooser.getSelectedFile());
				} catch (IOException ex) {
					System.err.println(ex);
					JOptionPane.showMessageDialog(self, ex.getLocalizedMessage(), "Error", JOptionPane.ERROR_MESSAGE);
				}
			}
		}
		
	}
	
	/**
	 * Controller for position buttons.
	 */
	private class PositionButtonController implements ActionListener {
		
		public void actionPerformed(ActionEvent e) {
			if (positions != null) {
				if ("< Previous".equals(e.getActionCommand())) {
					previousPosition();
				} else {
					nextPosition();
				}
			}
		}
		
	}
	
	
	/**
	 * Returns the position.
	 * 
	 * @return	the position
	 */
	public Point2D getPosition() {
		return new Point2D.Double((Double) positionX.getValue(), (Double) positionY.getValue());
	}
	
	/**
	 * Sets the position.
	 * 
	 * @param p	the position
	 */
	public void setPosition(Point2D p) {
		positionX.setValue(p.getX());
		positionY.setValue(p.getY());
	}
	
	/**
	 * Returns the angle.
	 * 
	 * @return	the last obtained angle or NaN if compass not in use
	 */
	public double getAngle() {
		if (!angle.isEnabled())
			return Double.NaN;
		try {
			return Double.parseDouble(angle.getText());
		} catch (Exception ex) {
			return Double.NaN;
		}
	}
	
	/**
	 * Sets the angle.
	 * 
	 * @param angle	the angle
	 */
	public void setAngle(double angle) {
		this.angle.setText("" + angle);
	}
	
	/**
	 * Enables or disables the compass.
	 * 
	 * @param enabled	whether to enable use of compass
	 */
	public void setCompassEnabled(boolean enabled) {
		//angle.setEnabled(enabled);
	}
	
	/**
	 * Sets the position file.
	 * 
	 * @param file	the file
	 * @throws IOException
	 */
	public void setPositionFile(File file) throws IOException {
		if (file == null) {
			positions = null;
			positionFile.setText("");
		} else {
			positions = PositionFileParser.parse(file);
			positionFile.setText(file.getAbsolutePath());
		}
		positionIndex = -1;
		nextPositionButton.setEnabled((positions != null) && (positions.size() > 0));
		previousPositionButton.setEnabled(false);
	}
	
	/**
	 * Advances the position.
	 */
	protected void nextPosition() {
		if ((positions == null) || (positionIndex >= positions.size()))
			throw new RuntimeException();
		Point2D p = positions.get(++positionIndex);
		setPosition(p);
		updatePositionButtons();
	}
	
	/**
	 * Returns to previous position.
	 */
	protected void previousPosition() {
		if ((positions == null) || (positionIndex <= 0))
			throw new RuntimeException();
		Point2D p = positions.get(--positionIndex);
		setPosition(p);
		updatePositionButtons();
	}
	
	/**
	 * Updates the position buttons.
	 */
	protected void updatePositionButtons() {
		nextPositionButton.setEnabled((positions != null) && (positionIndex < positions.size()));
		previousPositionButton.setEnabled((positions != null) && (positionIndex > 0));
	}
	
}
