/*
 * This file is part of Loctrace.
 * 
 * Copyright (c) 2007 Thomas King <king@informatik.uni-mannheim.de>,
 * University of Mannheim, Germany
 * 
 * All rights reserved.
 *
 * Loclib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Loclib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Loclib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package org.pi4.loctrace.wirelesslan;

import java.io.PrintStream;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.JOptionPane;

import org.pi4.loclib.wirelesslan.*;
import org.pi4.loctrace.TraceListener;
import org.pi4.loctrace.util.Counter;
import org.pi4.locutil1.MACAddress;


/**
 * A WLAN based tracing operation.
 * 
 * @author Andreas
 */
public class Trace implements Runnable {
	
	protected TraceListener listener;
	protected boolean cancel = false;
	protected PrintStream out = null;
	protected boolean onlyAPs = true;
	protected ScanMode scanMode = ScanMode.ACTIVE;
	protected boolean usePcap = false;
	protected int timeout = 250 * 1000;
	protected String netDevice = "eth0", rtapDevice = "rtap0";
	protected int maxScans = 0, maxReadings = 0;
	
	/**
	 * Create a new non-recording instance.
	 * 
	 * @param listener	a listener
	 */
	public Trace(TraceListener listener) {
		this.listener = listener;
	}
	
	/**
	 * Creates a new trace instance.
	 * 
	 * @param listener	a listener
	 * @param log	a loggin destination
	 */
	public Trace(TraceListener listener, PrintStream log) {
		this.listener = listener;
		this.out = log;
	}
	
	/**
	 * Sets the scan mode.
	 * @param scanMode	the scan mode
	 */
	public void setScanMode(ScanMode scanMode) {
		this.scanMode = scanMode;
	}
	
	/**
	 * Sets the use of pcap.
	 * @param usePcap	whether to use pcap
	 */
	public void setUsePcap(boolean usePcap) {
		this.usePcap = usePcap;
	}
	
	/**
	 * Sets the wireless network device.
	 * @param device	the device, e.g. eth0
	 */
	public void setNetDevice(String device) {
		this.netDevice = device;
	}
	
	/**
	 * Sets the pcap radiotap device
	 * @param device	the device, e.g. rtap0
	 */
	public void setRtapDevice(String device) {
		this.rtapDevice = device;
	}
	
	/**
	 * Sets the timeout.
	 * 
	 * @param timeout	the timeout in ms
	 */
	public void setTimeout(int timeout) {
		this.timeout = timeout;
	}
	
	/**
	 * Sets the maximum number of scans.
	 * 
	 * @param scans	the number of scans
	 */
	public void setMaxScans(int scans) {
		this.maxScans = scans;
	}
	
	/**
	 * Sets the maximum number of readings.
	 * 
	 * @param readings	the number of readings
	 */
	public void setMaxReadings(int readings) {
		this.maxReadings = readings;
	}
	
	/**
	 * Cancels tracing.
	 */
	public void cancel() {
		cancel = true;
	}
	
	/**
	 * Returns whether tracing was cancelled by the user.
	 * @return	true if cancelled, false otherwise
	 */
	public boolean isCancelled() {
		return cancel;
	}
	
	/**
	 * Performs the scan(s).
	 */
	public void run() {
		int scans = 0;
		Counter<String> readingsCounter = null;
		try {
			int packetCount = -1;
			MACAddress mac = Scan.getMAC(netDevice);
			if (listener != null)
				listener.traceInitiated();
			if (out != null) {
				if (listener != null)
					listener.printPreamble(out);
				if (!usePcap) {
					out.print("# iwlib based ");
					out.print((scanMode == ScanMode.PASSIVE) ? "passive" : "active");
					out.println(" scan");
				} else if (scanMode == ScanMode.MONITOR) {
					out.println("# monitor sniffing");
				} else if (scanMode == ScanMode.PASSIVE) {
					out.println("# pcap based passive scan");
				} else {
					out.println("# iwlib-initiated active scan using pcap");
				}
				out.print("# timeout: ");
				out.print(timeout);
				out.println(" ms");
				out.print("# max. scans: ");
				out.println(maxScans);
				out.print("# max. readings: ");
				out.println(maxReadings);
			}
			if (maxReadings != 0) {
				readingsCounter = new Counter<String>();
			}
			while (!cancel && ((maxScans == 0) || (scans < maxScans)) && ((maxReadings == 0) || (readingsCounter.getMax() < maxReadings))) {
				Vector results;
				if (!usePcap) {
					results = Scan.scan(netDevice, timeout, scanMode != ScanMode.PASSIVE);
				} else if (scanMode == ScanMode.MONITOR) {
					results = Scan.monitorSniffingScan(rtapDevice, packetCount, timeout);
				} else {
					results = Scan.pcapScan(netDevice, rtapDevice, packetCount, timeout, scanMode != ScanMode.PASSIVE);
				}
				scans++;
				if (out != null) {
					if (listener != null)
						out.print(listener.getPreamble());
					out.print(";id=");
					out.print(mac);
				}
				Hashtable<String, Integer> readings = new Hashtable<String, Integer>();
				Iterator it = results.iterator();
				while (it.hasNext()) {
					ScanResult result = (ScanResult) it.next();
					if ((out != null) && !(onlyAPs && (result.getMode() != 3))) {
						out.print(';');
						out.print(result.getAddress());
						out.print('=');
						out.print(result.getSignalStrength());
						out.print(',');
						out.print(result.getFrequency());
						out.print(',');
						out.print(result.getMode());
						out.print(',');
						out.print(result.getNoiseLevel());
					}
					if (result.getMode() == 3) {
						readings.put(result.getAddress().toString(), (int) result.getSignalStrength());
					}
				}
				if (out != null)
					out.println();
				if (listener != null)
					listener.traceResult(readings);
				for (String key: readings.keySet()) {
					readingsCounter.add(key);
				}
				Thread.sleep(400);
			}
			if (out != null) {
				if (cancel) {
					out.println("# Trace was cancelled by user.");
				} else if ((maxScans != 0) && (scans >= maxScans)) {
					out.println("# Trace finished after " + scans + " scans.");
				} else if ((maxReadings != 0) && (readingsCounter.getMax() >= maxReadings)) {
					out.println("# Trace finished after " + readingsCounter.getMax() + " readings.");
				}
				if (listener != null)
					listener.traceFinishing(out);
				out.flush();
			}
			if (listener != null)
				listener.traceFinished(!cancel);
		} catch (Throwable ex) {
			System.err.println(ex.getLocalizedMessage());
			if (out != null) {
				out.println("# trace aborted due to exception: " + ex.getMessage());
				out.flush();
			}
			JOptionPane.showMessageDialog(null, ex.getLocalizedMessage(), "Trace error", JOptionPane.ERROR_MESSAGE);
			if (listener != null)
				listener.traceFinished(false);
		}
	}
}
