/*
 * This file is part of Loctrace.
 * 
 * Copyright (c) 2007 Thomas King <king@informatik.uni-mannheim.de>,
 * University of Mannheim, Germany
 * 
 * All rights reserved.
 *
 * Loclib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Loclib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Loclib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package org.pi4.loctrace.wirelesslan;

import java.awt.Color;
import java.awt.Container;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.awt.geom.Point2D;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Hashtable;

import javax.swing.*;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;

import org.pi4.loclib.f350compassfd.CompassBoardReply;
import org.pi4.loctrace.Histogram;
import org.pi4.loctrace.PositionPanel;
import org.pi4.loctrace.TraceListener;
import org.pi4.loctrace.wirelesslan.txpower.TxPowerPanel;

/**
 * Main window for tracer application.
 * 
 * @author Andreas
 */
@SuppressWarnings("serial")
public class Tracer extends JFrame {
	
	// view
	protected JFrame frame = this;
	private PositionPanel positionPanel;
	private JButton traceButton, traceFileButton;
	private Histogram histogram;
	private JProgressBar progressBar;
	private WlanPanel wlan;
	private TxPowerPanel txPowerPanel;
	private JTextField traceFile;
	private JCheckBox log;
	
	// controller
	private TraceController traceController;
	
	// model
	private org.pi4.loclib.f350compassfd.InputOutputHandler compass;
	protected Trace trace;
	
	public Tracer() {
		setTitle("Tracer");
		setSize(1000, 1000);
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		addWindowListener(new WindowController());
		
		GridBagLayout mainLayout = new GridBagLayout();
		getContentPane().setLayout(mainLayout);
		
		setUpUI();
		setCompassEnabled(false);
		try {
			positionPanel.setPositionFile(null);
		} catch (IOException ex) {} // ...will not occur for argument null
		wlan.setWlanDevice("eth2");
		wlan.setPcapEnabled(false);
		wlan.setPcapDevice("rtap0");
		wlan.setScanMode(ScanMode.ACTIVE);
		wlan.setTimeout(300);
		wlan.setMaxScans(110);
		wlan.setMaxReadings(110);
		setTraceFile(new File("test.trace"));
		traceFile.addCaretListener(new TraceFileTextFieldController());
		setTraceEnabled(true);
		
		txPowerPanel.setVisible(false);
		
		traceController = new TraceController();
		
		//pack();
		setExtendedState(MAXIMIZED_BOTH);
		setVisible(true);
	}
	
	/**
	 * Sets up the GUI.
	 */
	private void setUpUI() {
		Container contentPane = getContentPane();
		GridBagConstraints constraints = new GridBagConstraints();
		positionPanel = new PositionPanel();
		constraints.anchor = GridBagConstraints.NORTHWEST;
		constraints.gridx = 0;
		constraints.gridy = 0;
		constraints.gridwidth = GridBagConstraints.REMAINDER;
		constraints.gridheight = 1;
		constraints.weightx = 1.0;
		constraints.weighty = 0.0;
		constraints.fill = GridBagConstraints.HORIZONTAL;
		contentPane.add(positionPanel, constraints);
		constraints.anchor = GridBagConstraints.CENTER;
		
		constraints.gridy++;
		wlan = new WlanPanel();
		contentPane.add(wlan, constraints);
		
		constraints.gridy++;
		txPowerPanel = new TxPowerPanel();
		contentPane.add(txPowerPanel, constraints);
		
		constraints.gridy++;
		constraints.gridwidth = 1;
		constraints.weightx = 0.0;
		constraints.fill = GridBagConstraints.NONE;
		log = new JCheckBox("Record ");
		log.addActionListener(new TraceEnabledController());
		contentPane.add(log, constraints);
		constraints.gridx++;
		constraints.gridwidth = GridBagConstraints.RELATIVE;
		constraints.weightx = 1.0;
		constraints.fill = GridBagConstraints.HORIZONTAL;
		traceFile = new JTextField();
		contentPane.add(traceFile, constraints);
		constraints.gridx++;
		constraints.gridwidth = 1;
		constraints.weightx = 0.0;
		constraints.fill = GridBagConstraints.NONE;
		traceFileButton = new JButton("Browse...");
		traceFileButton.addActionListener(new TraceFileButtonController());
		contentPane.add(traceFileButton, constraints);
		
		constraints.gridy++;
		constraints.gridx = 0;
		constraints.gridwidth = GridBagConstraints.REMAINDER;
		constraints.weightx = 1.0;
		constraints.fill = GridBagConstraints.HORIZONTAL;
		traceButton = new JButton("Trace");
		traceButton.addActionListener(new TraceButtonController());
		traceButton.setMnemonic('t');
		contentPane.add(traceButton, constraints);
		
		constraints.gridy++;
		constraints.weighty = 1.0;
		constraints.gridheight = GridBagConstraints.RELATIVE;
		constraints.fill = GridBagConstraints.BOTH;
		constraints.anchor =  GridBagConstraints.CENTER;
		histogram = new Histogram();
		contentPane.add(histogram, constraints);
		
		constraints.gridy++;
		constraints.weighty = 0.0;
		constraints.gridheight = 1;
		constraints.fill = GridBagConstraints.HORIZONTAL;
		constraints.anchor = GridBagConstraints.SOUTH;
		progressBar = new JProgressBar();
		progressBar.setStringPainted(true);
		contentPane.add(progressBar, constraints);
	}
	
	/**
	 * Handles entering the trace mode.
	 * Disables UI elements.
	 */
	private void enterTraceMode() {
		traceButton.setText("Cancel");
		
		positionPanel.setEnabled(false);
		wlan.setEnabled(false);
		log.setEnabled(false);
		traceFile.setEnabled(false);
		traceFileButton.setEnabled(false);
	}
	
	/**
	 * Handles leaving the trace mode.
	 * Re-enabled UI elements.
	 */
	private void leaveTraceMode() {
		positionPanel.setEnabled(true);
		wlan.setEnabled(true);
		log.setEnabled(true);
		setTraceEnabled(getTraceEnabled());
		
		traceButton.setText("Trace");
	}
	
	/**
	 * Initiates a scan.
	 */
	public void startTrace() {
		try {
			txPowerPanel.traceInitiated();
			enterTraceMode();
			PrintStream traceFile = null;
			if (getTraceEnabled())
				traceFile = new PrintStream(new FileOutputStream(getTraceFile(), true));
			trace = new Trace(traceController, traceFile);
			trace.setScanMode(wlan.getScanMode());
			trace.setUsePcap(wlan.isPcapEnabled());
			trace.setNetDevice(wlan.getWlanDevice());
			trace.setRtapDevice(wlan.getPcapDevice());
			trace.setTimeout(wlan.getTimeout());
			trace.setMaxScans(wlan.getMaxScans());
			trace.setMaxReadings(wlan.getMaxReadings());
			histogram.resetHistogram();
			histogram.showMeterBars(true);
			histogram.showRSSCounts(true);
			histogram.repaint();
			Thread t = new Thread(trace);
			t.start();
		} catch (Exception ex) {
			System.err.println(ex);
			leaveTraceMode();
		}
	}
	
	/**
	 * Cancels a running trace.
	 */
	public void cancelTrace() {
		if (trace != null)
			trace.cancel();
	}
	
	/*
	 * Controller
	 * 
	 */
	
	/**
	 * Controller for window
	 */
	private class WindowController implements WindowListener {

		public void windowActivated(WindowEvent e) {
		}

		public void windowClosed(WindowEvent e) {
			System.exit(0);
		}

		public void windowClosing(WindowEvent e) {
		}

		public void windowDeactivated(WindowEvent e) {
		}

		public void windowDeiconified(WindowEvent e) {
		}

		public void windowIconified(WindowEvent e) {
		}

		public void windowOpened(WindowEvent e) {
		}
		
	}
	
	/**
	 * Controller for trace file recording checkbox
	 */
	private class TraceEnabledController implements ActionListener {

		public void actionPerformed(ActionEvent e) {
			setTraceEnabled(log.isSelected());
		}
		
	}
	
	/**
	 * Controller for trace file text field
	 */
	public class TraceFileTextFieldController implements CaretListener {
		
		public void caretUpdate(CaretEvent e) {
			updateTraceFileColor();
		}
	}
	
	/**
	 * Controller for trace file button
	 */
	private class TraceFileButtonController implements ActionListener {

		public void actionPerformed(ActionEvent e) {
			JFileChooser chooser = new JFileChooser(traceFile.getText());
			if (chooser.showSaveDialog(frame) == JFileChooser.APPROVE_OPTION) {
				setTraceFile(chooser.getSelectedFile());
			}
		}
		
	}
	
	/**
	 * Controller for trace button
	 */
	private class TraceButtonController implements ActionListener {

		public void actionPerformed(ActionEvent e) {
			if ("Cancel".equals(e.getActionCommand())) {
				cancelTrace();
			} else {
				progressBar.setMinimum(0);
				progressBar.setMaximum(wlan.getMaxScans() * txPowerPanel.getAutoStepCount());
				progressBar.setValue(0);
				startTrace();
			}
		}
		
	}
	
	
	/**
	 * Controller for compass
	 */
	private class CompassController implements org.pi4.loclib.f350compassfd.CompassListener {

		public void handleCompassBoardReply(CompassBoardReply compassData) {
			double angle = compassData.getDirectionDegrees() + (compassData.getDirectionMinutes() / 60);
			positionPanel.setAngle(angle);
		}
		
	}
	
	/**
	 * Controller for trace
	 */
	// there is only one instance!
	private class TraceController implements TraceListener {
		
		private int initial, count;
		
		public void traceInitiated() {
			initial = progressBar.getValue();
			count = 0;
		}
		
		public void printPreamble(PrintStream out) {
			Date date = new Date();
			SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
			out.print("# trace started ");
			out.println(df.format(date));
			// tx power?
		}
		
		public String getPreamble() {
			StringBuilder sb = new StringBuilder();
			sb.append("t=");
			sb.append(System.currentTimeMillis());
			Point2D pos = positionPanel.getPosition();
			sb.append(";pos=");
			sb.append(pos.getX());
			sb.append(',');
			sb.append(pos.getY());
			sb.append(',');
			sb.append(0.0);
			if (!Double.isNaN(positionPanel.getAngle())) {
				sb.append(";degree=");
				sb.append(positionPanel.getAngle());
			}
			String txp = txPowerPanel.getPreamble();
			if ((txp != null) && (txp.length() > 0)) {
				if (txp.charAt(0) != ';')
					sb.append(';');
				sb.append(txp);
			}
			return sb.toString();
		}
		
		public void traceResult(Hashtable<String, Integer> reading) {
			count++;
			histogram.addToHistory(reading);
			histogram.repaint();
			progressBar.setValue(initial + count);
			//System.out.println("after scan: " + progressBar.getValue());
		}
		
		public void traceFinishing(PrintStream out) {
			//
		}
		
		public void traceFinished(boolean success) {
			progressBar.setValue(initial + wlan.getMaxScans());
			//System.out.println("after finish: " + progressBar.getValue());
			leaveTraceMode();
			if (!txPowerPanel.traceFinished(success)) {
				startTrace();
			} else {
				java.awt.Toolkit.getDefaultToolkit().beep();
				progressBar.setValue(0);
			}
		}
		
	}
	
	// data access
	
	/**
	 * Enables or disables the use of a compass.
	 * @param enabled	whether the compass shall be used
	 */
	public void setCompassEnabled(boolean enabled) {
		if (enabled) {
			if (compass == null) {
				try {
					compass = org.pi4.loclib.f350compassfd.InputOutputHandler.getInstance(new File("/dev/ttyUSB0"), false);
					compass.addCompassListener(new CompassController());
				} catch (IOException ex) {
					enabled = false;
				}
			}
		} else {
			if (compass != null) {
				compass.close();
				compass = null;
			}
		}
		positionPanel.setCompassEnabled(enabled);
	}
	
	/**
	 * Gets whether the trace is recorded.
	 * @return	true if recorded to trace file, false otherwise
	 */
	public boolean getTraceEnabled() {
		return log.isSelected();
	}
	
	/**
	 * Enables or disables recording of trace file.
	 * @param enabled	true if trace file shall be generated, false otherwise
	 */
	public void setTraceEnabled(boolean enabled) {
		log.setSelected(enabled);
		traceFile.setEnabled(enabled);
		traceFileButton.setEnabled(enabled);
	}
	
	/**
	 * Gets the trace file.
	 * @return	the trace file
	 */
	public File getTraceFile() {
		return new File(traceFile.getText());
	}
	
	/**
	 * Sets the trace file.
	 * @param file	the file
	 */
	public void setTraceFile(File file) {
		traceFile.setText(file.getPath());
		updateTraceFileColor();
	}
	
	/**
	 * Updates the color of the trace file path depending on whether the file exists.
	 */
	protected void updateTraceFileColor() {
		File file = getTraceFile();
		traceFile.setForeground(file.exists() ? Color.GREEN : Color.RED);
	}
	
	
	/**
	 * Starts up the tracer GUI.
	 * 
	 * @param args	the command line arguments
	 */
	public static void main(String[] args) {
		new Tracer();
	}

}
