/*
 * This file is part of Loctrace.
 * 
 * Copyright (c) 2007 Thomas King <king@informatik.uni-mannheim.de>,
 * University of Mannheim, Germany
 * 
 * All rights reserved.
 *
 * Loclib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Loclib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Loclib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package org.pi4.loctrace.wirelesslan;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.*;

/**
 * Configuration panel for WLAN.
 * 
 * @author Andreas
 */
@SuppressWarnings("serial")
class WlanPanel extends JPanel {
	private JTextField wlanDevice;
	private JSpinner wlanTimeout, wlanMaxScans, wlanMaxReadings;
	private JCheckBox wlanPcap;
	private JTextField wlanPcapDevice;
	private JComboBox wlanScanMode;
	
	public WlanPanel() {
		this.setBorder(BorderFactory.createTitledBorder("WLAN"));
		GridBagLayout wlanLayout = new GridBagLayout();
		this.setLayout(wlanLayout);
		
		final int h = 24;
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.gridx = 0;
		constraints.gridy = 0;
		constraints.weightx = 0.0;
		constraints.weighty = 1.0;
		constraints.anchor = GridBagConstraints.WEST;
		constraints.insets.set(8, 8, 0, 8);
		JLabel label = new JLabel("Device:");
		this.add(label, constraints);
		constraints.gridx++;
		constraints.insets.set(5, 0, 0, 8);
		wlanDevice = new JTextField(5);
		this.add(wlanDevice, constraints);
		
		constraints.gridy++;
		constraints.gridx = 0;
		constraints.insets.set(8, 8, 0, 8);
		wlanPcap = new JCheckBox("Use pcap");
		wlanPcap.addActionListener(new PcapSelectionController());
		this.add(wlanPcap, constraints);
		constraints.gridx++;
		constraints.insets.set(8, 0, 0, 8);
		label = new JLabel("Radiotap device:");
		this.add(label, constraints);
		constraints.gridx++;
		constraints.weightx = 1.0;
		constraints.insets.set(5, 0, 0, 0);
		wlanPcapDevice = new JTextField(5);
		this.add(wlanPcapDevice, constraints);
		
		constraints.gridy++;
		constraints.gridx = 0;
		constraints.weightx = 0;
		constraints.insets.set(8, 8, 0, 8);
		label = new JLabel("Scan mode:");
		this.add(label, constraints);
		constraints.gridx++;
		constraints.insets.set(5, 0, 0, 0);
		wlanScanMode = new JComboBox(new String[] { "active", "passive", "monitor" });
		wlanScanMode.addActionListener(new ScanModeController());
		this.add(wlanScanMode, constraints);
		
		constraints.gridy++;
		constraints.gridx = 0;
		constraints.insets.set(8, 8, 0, 0);
		label = new JLabel("Timeout:");
		this.add(label, constraints);
		constraints.gridx++;
		constraints.insets.set(5, -8, 0, 0);
		JPanel panel = new JPanel();
		FlowLayout layout = new FlowLayout(FlowLayout.LEADING, 8, 0);
		panel.setLayout(layout);
		wlanTimeout = new JSpinner(new SpinnerNumberModel(250, 0, Integer.MAX_VALUE, 1));
		wlanTimeout.setPreferredSize(new Dimension(60, h));
		panel.add(wlanTimeout);
		panel.add(new JLabel("ms"));
		this.add(panel, constraints);
		
		constraints.gridy++;
		constraints.gridx = 0;
		constraints.insets.set(8, 8, 0, 8);
		label = new JLabel("Max. number of scans:");
		this.add(label, constraints);
		constraints.gridx++;
		constraints.insets.set(5, 0, 0, 0);
		wlanMaxScans = new JSpinner(new SpinnerNumberModel(110, 0, Integer.MAX_VALUE, 1));
		wlanMaxScans.setPreferredSize(new Dimension(60, h));
		this.add(wlanMaxScans, constraints);
		
		constraints.gridy++;
		constraints.gridx = 0;
		constraints.insets.set(8, 8, 8, 8);
		label = new JLabel("Max. number of readings per AP:");
		this.add(label, constraints);
		constraints.gridx++;
		constraints.insets.set(5, 0, 5, 0);
		wlanMaxReadings = new JSpinner(new SpinnerNumberModel(110, 0, Integer.MAX_VALUE, 1));
		wlanMaxReadings.setPreferredSize(new Dimension(60, h));
		this.add(wlanMaxReadings, constraints);
	}
	
	public void setEnabled(boolean enabled) {
		super.setEnabled(enabled);
		wlanDevice.setEnabled(enabled);
		wlanPcap.setEnabled(enabled);
		wlanScanMode.setEnabled(enabled);
		wlanTimeout.setEnabled(enabled);
		wlanMaxScans.setEnabled(enabled);
		wlanMaxReadings.setEnabled(enabled);
		if (enabled) {
			setPcapEnabled(isPcapEnabled());
		} else {
			wlanPcapDevice.setEnabled(false);
		}
	}
	
	/**
	 * Controller for pcap checkbox.
	 */
	private class PcapSelectionController implements ActionListener {
		
		public void actionPerformed(ActionEvent e) {
			wlanPcapDevice.setEnabled(wlanPcap.isSelected());
		}
		
	}
	
	/**
	 * Controller for scan mode drop-down box.
	 */
	private class ScanModeController implements ActionListener {
		
		public void actionPerformed(ActionEvent e) {
			ScanMode scanMode = getScanMode();
			wlanPcap.setEnabled(scanMode != ScanMode.MONITOR);
			if (scanMode == ScanMode.MONITOR)
				setPcapEnabled(true);
		}
		
	}
	
	/**
	 * Sets the WLAN device.
	 * 
	 * @param device	the device, e.g. eth0
	 */
	public void setWlanDevice(String device) {
		wlanDevice.setText(device);
	}
	
	/**
	 * Returns the WLAN device.
	 * 
	 * @return	the device name
	 */
	public String getWlanDevice() {
		return wlanDevice.getText();
	}
	
	/**
	 * Sets the pcap radiotap device.
	 * 
	 * @param device	the device, e.g. rtap0
	 */
	public void setPcapDevice(String device) {
		wlanPcapDevice.setText(device);
	}
	
	/**
	 * Returns the pcap radiotap device.
	 * 
	 * @return	the device name
	 */
	public String getPcapDevice() {
		return wlanPcapDevice.getText();
	}
	
	/**
	 * Enables or disables use of pcap.
	 * 
	 * @param enabled	whether to use pcap
	 */
	public void setPcapEnabled(boolean enabled) {
		wlanPcap.setSelected(enabled);
		wlanPcapDevice.setEnabled(enabled);
	}
	
	/**
	 * Returns whether pcap shall be used.
	 * 
	 * @return	true if pcap shall be used, false otherwise
	 */
	public boolean isPcapEnabled() {
		return wlanPcap.isSelected();
	}
	
	/**
	 * Sets the timeout.
	 * 
	 * @param timeout	the timeout in ms
	 */
	public void setTimeout(int timeout) {
		wlanTimeout.setValue(timeout);
	}
	
	/**
	 * Returns the timeout.
	 * 
	 * @return	the timeout
	 */
	public int getTimeout() {
		return (Integer) wlanTimeout.getValue();
	}
	
	/**
	 * Sets the maximum number of scans.
	 * 
	 * @param maxRepetitions	the number of scans
	 */
	public void setMaxScans(int maxRepetitions) {
		wlanMaxScans.setValue(maxRepetitions);
	}
	
	/**
	 * Returns the maximum number of scans.
	 * 
	 * @return	the number of scans
	 */
	public int getMaxScans() {
		return (Integer) wlanMaxScans.getValue();
	}
	
	/**
	 * Sets the maximum number of readings.
	 * 
	 * @param maxReadings	the number of readings
	 */
	public void setMaxReadings(int maxReadings) {
		wlanMaxReadings.setValue(maxReadings);
	}
	
	/**
	 * Returns the maximum number of readings.
	 * 
	 * @return	the number of readings
	 */
	public int getMaxReadings() {
		return (Integer) wlanMaxReadings.getValue();
	}
	
	/**
	 * Sets the scan mode.
	 * 
	 * @param scanMode	the scan mode
	 */
	public void setScanMode(ScanMode scanMode) {
		int index;
		if (scanMode == ScanMode.ACTIVE) {
			index = 0;
		} else if (scanMode == ScanMode.PASSIVE) {
			index = 1;
		} else if (scanMode == ScanMode.MONITOR) {
			index = 2;
		} else throw new RuntimeException();
		wlanScanMode.setSelectedIndex(index);
	}
	
	/**
	 * Returns the scan mode.
	 * 
	 * @return	the scan mode
	 */
	public ScanMode getScanMode() {
		switch (wlanScanMode.getSelectedIndex()) {
		case 0:
			return ScanMode.ACTIVE;
		case 1:
			return ScanMode.PASSIVE;
		case 2:
			return ScanMode.MONITOR;
		default:
			throw new RuntimeException();
		}
	}
}
