/*
 * This file is part of Loctrace.
 * 
 * Copyright (c) 2007 Thomas King <king@informatik.uni-mannheim.de>,
 * University of Mannheim, Germany
 * 
 * All rights reserved.
 *
 * Loclib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Loclib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Loclib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package org.pi4.loctrace.wirelesslan.txpower;

import java.io.IOException;
import java.io.OutputStreamWriter;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.*;

/**
 * Core functionality for switching the transmission power on wireless LAN access points
 * 
 * @author lemelson
 */
public class APPowerSwitcher {
	
	/**
	 * Switches the wireless LAN powerlevel of a WRT54g access point or router to a given value.
	 * 
	 * This method only works with APs/routers running DD-WRTv23SP1.
	 * 
	 * @param host The hostname or IP address
	 * @param user The username for administrative login
	 * @param password The user's password
	 * @param powerLevel Powerlevel you want to set the router to (valid range is 1 to 251)
	 * 
	 * @return	true if successful, false otherwise
	 */
	public static void switchPowerOnDD_WRT_v23_SP1(String host, String user, String password, int powerLevel) throws IOException { 
        boolean test = false; // set to true to test the gui without the need to have real APs available
		if (!test) {
	        try {
	        	// this is the router-specific string to switch the power via the web-interface
				String params = "submit_button=Wireless_Advanced&action=Apply&commit=1&txpwr=" + powerLevel;
				URL url = new URL("http", host, "/apply.cgi");
				HttpURLConnection con = (HttpURLConnection)url.openConnection();
				
				con.setConnectTimeout(1000);
				con.setDoInput(true);
				con.setDoOutput(true);
			    con.setRequestProperty("Authorization", encodeBasic(user, password));
			    con.connect();
			    
			    OutputStreamWriter out = new OutputStreamWriter(con.getOutputStream());
		        out.write(params);
		        out.flush();
		        out.close();
			    if (con.getResponseCode() != 200)
					throw new IOException("HTTP status " + con.getResponseCode() + ": " + con.getResponseMessage());
			} catch (java.net.MalformedURLException e) {
				System.err.println("Malformed URL:\n" + e.getLocalizedMessage());
				throw new RuntimeException("URL issue");
			}
		} else {
    		System.out.println("WARNING: TEST MODE ENABLED!" + "\n" + host.toUpperCase() + " NOT REALLY SWITCHED!");
			try {
				Thread.sleep(1000);
			} catch (InterruptedException iE) {
				System.err.println("Error while trying to suspend switcher-thread during test-mode operation!\n" + 
									iE.getMessage());
			}
			// return a random boolean value to also test the behaviour if error occurs
			if (!(new Random()).nextBoolean())
				throw new IOException("Test failed.");
		}
	}
	
	/**
	 * Encode the supplied username and password for HTTP Basic authorization.
	 * 
	 * This is done as we need to supply the username and password in this format to webservers using HTTP Basic authentication method.
	 * 
	 * @param username username to encode
	 * @param password password to encode
	 * @return <code>String</code> containing the Base64-encoded username and password
	 */
	private static String encodeBasic(String username, String password) {
		String s = username + ":" + password;
		String encs = new sun.misc.BASE64Encoder().encode(s.getBytes());
		return "Basic " + encs;
	}
}
