/*
 * This file is part of Loctrace.
 * 
 * Copyright (c) 2007 Thomas King <king@informatik.uni-mannheim.de>,
 * University of Mannheim, Germany
 * 
 * All rights reserved.
 *
 * Loclib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Loclib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Loclib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package org.pi4.loctrace.wirelesslan.txpower;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

/**
 * Configuration panel for tx power.
 * 
 * @author Andreas
 */
@SuppressWarnings("serial")
public class TxPowerPanel extends JPanel {
	
	private JSpinner power, minPower, maxPower, powerStep;
	private JButton applyPower;
	private JList apList;
	private JCheckBox autoSwitch;
	private JLabel autoToLabel, autoByLabel;
	private JPanel self;
	
	// model
	protected int lastPowerLevel = 0;
	protected Iterator<Integer> autoPower;
	
	public TxPowerPanel() {
		this.setBorder(BorderFactory.createTitledBorder("Tx Power"));
		GridBagLayout layout = new GridBagLayout();
		this.setLayout(layout);
		GridBagConstraints constraints = new GridBagConstraints();
		
		constraints.anchor = GridBagConstraints.WEST;
		constraints.gridy = 0;
		constraints.gridx = 0;
		constraints.gridheight = GridBagConstraints.REMAINDER;
		apList = new JList(new String[] { "julia", "titania", "rosalind", "miranda", "portia" });
		apList.setVisibleRowCount(5);
		JScrollPane scrollPane = new JScrollPane(apList);
		apList.addListSelectionListener(new APListSelectionController());
		this.add(scrollPane, constraints);
		
		constraints.gridx++;
		constraints.gridheight = 1;
		JLabel label = new JLabel("Set Tx power:");
		this.add(label, constraints);
		constraints.gridx++;
		power = new JSpinner(new SpinnerNumberModel(28, 1, 251, 1));
		this.add(power, constraints);
		constraints.gridx++;
		label = new JLabel(" mW");
		this.add(label, constraints);
		constraints.gridx++;
		constraints.gridwidth = 2;
		applyPower = new JButton("Apply");
		applyPower.setEnabled(false);
		applyPower.addActionListener(new ApplyButtonController());
		this.add(applyPower, constraints);
		constraints.weightx = 0;
		
		constraints.gridy++;
		constraints.gridx = 1;
		constraints.gridwidth = 3;
		autoSwitch = new JCheckBox("Switch automatically from ", false);
		autoSwitch.setEnabled(false);
		autoSwitch.addActionListener(new AutoSwitchCheckboxController());
		this.add(autoSwitch, constraints);
		constraints.gridx += 3;
		constraints.gridwidth = 1;
		minPower = new JSpinner(new SpinnerNumberModel(1, 1, 251, 1));
		minPower.setEnabled(false);
		minPower.addChangeListener(new AutoMinMaxPowerController());
		this.add(minPower, constraints);
		constraints.gridx++;
		autoToLabel = new JLabel(" to ");
		autoToLabel.setEnabled(false);
		this.add(autoToLabel, constraints);
		constraints.gridx++;
		maxPower = new JSpinner(new SpinnerNumberModel(251, 1, 251, 1));
		maxPower.setEnabled(false);
		maxPower.addChangeListener(new AutoMinMaxPowerController());
		this.add(maxPower, constraints);
		constraints.gridx++;
		autoByLabel = new JLabel(" by ");
		autoByLabel.setEnabled(false);
		this.add(autoByLabel, constraints);
		constraints.gridx++;
		constraints.weightx = 1.0;
		powerStep = new JSpinner(new SpinnerNumberModel(1, 1, 250, 1));
		powerStep.setEnabled(false);
		this.add(powerStep, constraints);
	}
	
	// model
	
	/**
	 * Retrieves a list of selected APs.
	 */
	public String[] getAPs() {
		Object[] selection = apList.getSelectedValues();
		String[] values = new String[selection.length];
		for (int i = 0; i < selection.length; i++) {
			values[i] = (String) selection[i];
			if (values[i].indexOf('.') == -1)
				values[i] = values[i] + ".informatik.uni-mannheim.de";
		}
		return values;
	}
	
	/**
	 * Enables or disables auto-switching.
	 * 
	 * @param enabled
	 */
	protected void setAutoEnabled(boolean enabled) {
		autoSwitch.setEnabled(enabled);
		autoToLabel.setEnabled(enabled);
		autoByLabel.setEnabled(enabled);
		minPower .setEnabled(enabled && autoSwitch.isSelected());
		maxPower .setEnabled(enabled && autoSwitch.isSelected());
		powerStep.setEnabled(enabled && autoSwitch.isSelected());
	}
	
	/**
	 * Determines whether auto-switching is activated.
	 * 
	 * @return	true if activated, false otherwise
	 */
	public boolean isAutoActivated() {
		return autoSwitch.isSelected();
	}
	
	/**
	 * Retrieves a new iterator over the tx power values.
	 * 
	 * @return	an interator
	 */
	public Iterator<Integer> autoPowerIterator() {
		return new PowerIterator();
	}
	
	/**
	 * Determines how many distinct tx power values will be used.
	 * 
	 * @return	number of steps
	 */
	public int getAutoStepCount() {
		int n = 1;
		if ((Integer)powerStep.getValue() > 0)
			n += ((Integer)maxPower.getValue() - (Integer)minPower.getValue()) / (Integer)powerStep.getValue();
		return n;
	}
	
	/**
	 * Sets tx power for a single AP.
	 * 
	 * @param host	the host name
	 * @param powerLevel	the desired tx power level
	 * @throws IOException
	 */
	private void setTxPower(String host, int powerLevel) throws IOException {
		APPowerSwitcher.switchPowerOnDD_WRT_v23_SP1(host, "root", "linksys", powerLevel);
	}
	
	/**
	 * Sets tx power on all selected APs to the next value.
	 */
	public void setAutoTxPower() {
		try {
			Integer power = autoPower.next();
			System.out.println("Setting tx power: " + power);
			String[] hosts = getAPs();
			for (String host: hosts) {
				setTxPower(host, power);
			}
			lastPowerLevel = power;
		} catch (Exception ex) {
			JOptionPane.showMessageDialog(self, "Error while setting power level. Aborting.");
			autoPower = null;
			throw new RuntimeException(ex);
		}
	}
	
	/**
	 * A trace was initiated.
	 */
	public void traceInitiated() {
		if (isAutoActivated()) {
			if (autoPower == null)
				autoPower = autoPowerIterator();
			setAutoTxPower();
		}
	}
	
	/**
	 * Returns a preamble for each line of the trace file.
	 * 
	 * @return	string
	 */
	public String getPreamble() {
		if (lastPowerLevel > 0) {
			return "txp=" + lastPowerLevel;
		}
		return null;
	}
	
	/**
	 * A trace has finished.
	 * 
	 * @param success	whether the trace finished successfully
	 * @return	true if the series of traces is finished, false if a new scan should be initiated
	 */
	public boolean traceFinished(boolean success) {
		if (success && (autoPower != null) && autoPower.hasNext()) {
			return false; // -> next trace
		} else {
			autoPower = null;
		}
		return true; // done
	}
	
	// controllers
	
	/**
	 * Controller for AP selection
	 */
	private class APListSelectionController implements ListSelectionListener {

		public void valueChanged(ListSelectionEvent e) {
			boolean b = apList.getSelectedIndex() != -1;
			applyPower.setEnabled(b);
			setAutoEnabled(b);
		}
		
	}
	
	/**
	 * Controller for auto-switching checkbox
	 */
	private class AutoSwitchCheckboxController implements ActionListener {

		public void actionPerformed(ActionEvent e) {
			boolean b = autoSwitch.isSelected();
			minPower.setEnabled(b);
			maxPower.setEnabled(b);
			powerStep.setEnabled(b);
		}
		
	}
	
	/**
	 * Controller for manual tx power setting
	 */
	private class ApplyButtonController implements ActionListener {

		public void actionPerformed(ActionEvent e) {
			int value = (Integer) power.getValue();
			String[] hosts = getAPs();
			//JOptionPane.showMessageDialog(null, "Will be setting " + hosts.length + " APs to " + value + " mW Tx power now.");
			ArrayList<String> failedHosts = new ArrayList<String>();
			for (String host: hosts) {
				try {
					setTxPower(host, value);
				} catch (Exception ex) {
					failedHosts.add(host);
				}
			}
			lastPowerLevel = value;
			if (failedHosts.size() > 0) {
				StringBuffer sb = new StringBuffer("The following APs failed:\n");
				for (String host: failedHosts) {
					sb.append(host + "\n");
				}
				JOptionPane.showMessageDialog(self, sb.toString(), "Error", JOptionPane.WARNING_MESSAGE);
			}
		}
	}
	
	/**
	 * Controller for adjusting minimum and maximum of start, end and step values
	 */
	private class AutoMinMaxPowerController implements ChangeListener {

		public void stateChanged(ChangeEvent e) {
			int min = (Integer) minPower.getValue();
			int max = (Integer) maxPower.getValue();
			SpinnerNumberModel model;
			if (e.getSource() == minPower) {
				model = (SpinnerNumberModel) maxPower.getModel();
				model.setMinimum(min);
				if ((Integer)maxPower.getValue() < (Integer)model.getMinimum())
					model.setValue(model.getMinimum());
			} else if (e.getSource() == maxPower) {
				model = (SpinnerNumberModel) minPower.getModel();
				model.setMaximum(max);
				if ((Integer)minPower.getValue() > (Integer)model.getMaximum())
					model.setValue(model.getMaximum());
			}
			model = (SpinnerNumberModel) powerStep.getModel();
			model.setMaximum(max - min);
			if ((Integer)model.getValue() > (max - min))
				model.setValue(max - min);
		}
		
	}
	
	// helpers
	
	/**
	 * An iterator over the tx power values for auto-switching
	 */
	private class PowerIterator implements Iterator<Integer> {

		private Integer value = -1;
		
		public boolean hasNext() {
			return value < (Integer)maxPower.getValue();
		}
		
		public Integer next() {
			if (value < (Integer)minPower.getValue())
				return value = (Integer)minPower.getValue();
			value += (Integer)powerStep.getValue();
			if (value <= (Integer)maxPower.getValue()) // if last+step > max then max will not be reached
				return value;
			return null;
		}

		public void remove() {
			throw new RuntimeException("Not implemented");
		}
		
	}
	
}
